package jabadot;

import java.io.*;
import java.util.*;
import java.sql.SQLException;	// Only used by JDBC version
import java.lang.reflect.*;		// For loading our subclass class.

/** A base for several "database" accessors for User objects.
 * We use a Singleton access method for efficiency and to enforce
 * single access on the database, which means we can keep an in-memory
 * copy (in an ArrayList) perfectly in synch with the database.
 *
 * We provide field numbers, which are 1-based (for SQL), not 0 as per Java.
 */
public abstract class UserDB {

	public static final int NAME		= 1;
	public static final int PASSWORD	= 2;
	public static final int FULLNAME 	= 3;
	public static final int EMAIL		= 4;
	public static final int CITY		= 5;
	public static final int PROVINCE	= 6;
	public static final int COUNTRY		= 7;
	public static final int PRIVS		= 8;

	protected ArrayList users;

	protected static UserDB singleton;

	/** Static code block to intialize the Singleton. */
	static {
		String dbClass = null;
		try {
			dbClass = JDConstants.getProperty("jabadot.userdb.class");
			singleton = (UserDB)Class.forName(dbClass).newInstance();
		} catch (ClassNotFoundException ex) {
			System.err.println("Unable to instantiate UserDB singleton " + 
				dbClass + " (" + ex.toString() + ")");
			throw new IllegalArgumentException(ex.toString());
		} catch (Exception ex) {
			System.err.println(
			"Unexpected exception: Unable to initialize UserDB singleton");
			ex.printStackTrace(System.err);
			throw new IllegalArgumentException(ex.toString());
		}
	}

	/** In some subclasses the constructor will probably load the database,
	 *  while in others it may defer this until getUserList().
	 */
	protected UserDB() throws IOException, SQLException {
		users = new ArrayList();
	}

	/** "factory" method to get an instance, which will always be
	 * the Singleton.
	 */
	public static UserDB getInstance() {
		if (singleton == null)
			throw new IllegalStateException("UserDB initialization failed");
		return singleton;
	}

	/** Get the list of users. */
	public ArrayList getUserList() {
		return users;
	}

	/** Get the User object for a given nickname */
	public User getUser(String nick) {
		Iterator it = users.iterator();
		while (it.hasNext()) {
			User u = (User)it.next();
			if (u.getName().equals(nick))
				return u;
		}
		return null;
	}

	public synchronized void addUser(User nu) throws IOException, SQLException {
		// Add it to the in-memory list
		users.add(nu);

		// Add it to the on-disk version
		// N.B. - must be done in subclass.
	}

	public abstract void setPassword(String nick, String newPass) 
	throws SQLException;

	public abstract void deleteUser(String nick)
	throws SQLException;
}
